/* Copyright (c) 2000 Boulder Real Time Technologies, Inc. */
/* All rights reserved */

#ifndef __brttutil__
#define __brttutil__

/* This software module is wholly owned by Boulder Real Time 
   Technologies, Inc. Any use of this software module without
   express written permission from Boulder Real Time Technologies,
   Inc. is prohibited. */

#include <synch.h>
#include <pthread.h>

#include "stock.h"

#ifdef  __cplusplus
extern "C" {
#endif

/* defines for parse_param */
#define	P_SINT	0	/* returns short int */
#define	P_LINT	1	/* returns long int */
#define	P_FLT	2	/* returns float */
#define	P_DBL	3	/* returns double */
#define	P_STR	4	/* returns string */
#define	P_PFTBL	5	/* returns Tbl with pf entries */
#define	P_TBL	6	/* returns Tbl with string entries */
#define	P_TBLPF	7	/* returns pf corresponding to a table input */
#define	P_PFARR	8	/* returns Arr with pf entries */
#define	P_ARR	9	/* returns Arr with string entries */
#define	P_ARRPF	10	/* returns pf corresponding to an array input */
#define	P_BOOL	11	/* returns boolean (int) */

/* return codes for pmtfifo_pop */
#define	PMTFIFO_ERROR		(-1)	/* error return */
#define	PMTFIFO_RELEASED	(0)	/* no data available and block released*/
#define	PMTFIFO_OK		(1)	/* data returned */
#define	PMTFIFO_TIMEOUT		(2)	/* timeout with no data available */
#define	PMTFIFO_NODATA		(3)	/* no data available on non-blocking FIFO */

/* typedefs for mtfifo */
typedef struct mtfifoelement_ {
	void *ptr;
	struct mtfifoelement_ *next;
	struct mtfifoelement_ *prev;
} MtfifoElement;

typedef struct mtfifo_ {
	mutex_t mutex;
	cond_t queue_condition;
	cond_t block_condition;
	int maxqueue;
	int queue;
	int block;
	int check;
	int release_block;
	MtfifoElement *head;
	MtfifoElement *tail;
} Mtfifo;

typedef struct pmtfifo_ {
	pthread_mutex_t mutex;
	pthread_cond_t queue_condition;
	pthread_cond_t block_condition;
	int maxqueue;
	int queue;
	int block;
	int check;
	int release_block;
	double timeout;
	MtfifoElement *head;
	MtfifoElement *tail;
} Pmtfifo;

int splitlist (char *list, int *argc, char ***argv);
int splitlistc (char *list, int *argc, char ***argv, char *schars);
void freeargs (int argc, char **argv);

int parse_param (Pf *pf, char *name, int type, int need, void *val);

Mtfifo *mtfifo_create (int maxqueue, int block, int check);
int mtfifo_destroy (Mtfifo *mtf, void (*free_value) ());
int mtfifo_push (Mtfifo *mtf, void *ptr);
int mtfifo_unpop (Mtfifo *mtf, void *ptr);
int mtfifo_pop (Mtfifo *mtf, void **ptr);
int mtfifo_data_available (Mtfifo *mtf);
int mtfifo_release_block (Mtfifo *mtf);

Pmtfifo *pmtfifo_create (int maxqueue, int block, int check);
int pmtfifo_set_timeout (Pmtfifo *mtf, double timeout);
int pmtfifo_destroy (Pmtfifo *mtf, void (*free_value) ());
int pmtfifo_push (Pmtfifo *mtf, void *ptr);
int pmtfifo_unpop (Pmtfifo *mtf, void *ptr);
int pmtfifo_pop (Pmtfifo *mtf, void **ptr);
int pmtfifo_data_available (Pmtfifo *mtf);
int pmtfifo_release_block (Pmtfifo *mtf);
int pmtfifo_release_blocks (Pmtfifo *mtf);


#ifdef  __cplusplus
}
#endif

#endif
